package com.arubanetworks.meridiansamples;

import static com.arubanetworks.meridiansamples.Application.APP_KEY;
import static com.arubanetworks.meridiansamples.Application.MAP_KEY;
import static com.arubanetworks.meridiansamples.Application.PLACEMARK_UID;

import android.Manifest;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.graphics.Color;
import android.graphics.Matrix;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.Fragment;
import androidx.core.content.ContextCompat;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;

import com.android.volley.VolleyError;
import com.arubanetworks.meridian.maps.directions.DirectionsOptions;
import com.arubanetworks.meridian.util.Strings;
import com.arubanetworks.meridian.location.LocationRequest;
import com.arubanetworks.meridian.location.MeridianLocation;
import com.arubanetworks.meridian.location.MeridianOrientation;
import com.arubanetworks.meridian.maps.MapFragment;
import com.arubanetworks.meridian.maps.MapOptions;
import com.arubanetworks.meridian.maps.MapView;
import com.arubanetworks.meridian.maps.Marker;

import org.acra.ACRA;

public class MainActivity extends AppCompatActivity {

    private DrawerLayout drawerLayout;
    private ListView drawerList;
    private ActionBarDrawerToggle drawerToggle;

    private CharSequence drawerTitle;
    private CharSequence title;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        title = drawerTitle = getTitle();
        drawerLayout = findViewById(R.id.drawer_layout);
        drawerList = findViewById(R.id.left_drawer);

        // set a custom shadow that overlays the main content when the drawer opens
        drawerLayout.setDrawerShadow(R.drawable.drawer_shadow, GravityCompat.START);
        // set up the drawer's list view with items and click listener
        drawerList.setAdapter(new ArrayAdapter<>(this, R.layout.drawer_list_item, getResources().getStringArray(R.array.section_titles)));
        drawerList.setOnItemClickListener(new DrawerItemClickListener());

        // enable ActionBar app icon to behave as action to toggle nav drawer
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setHomeButtonEnabled(true);
        }

        // ActionBarDrawerToggle ties together the the proper interactions
        // between the sliding drawer and the action bar app icon
        drawerToggle = new ActionBarDrawerToggle(
                this,                      /* host Activity */
                drawerLayout,              /* DrawerLayout object */
                R.string.navigation_drawer_open,  /* "open drawer" description for accessibility */
                R.string.navigation_drawer_close  /* "close drawer" description for accessibility */
        ) {
            public void onDrawerClosed(View view) {
                if (getSupportActionBar() != null) {
                    getSupportActionBar().setTitle(title);
                }
                getDelegate().invalidateOptionsMenu(); // creates call to onPrepareOptionsMenu()
            }

            public void onDrawerOpened(View drawerView) {
                if (getSupportActionBar() != null) {
                    getSupportActionBar().setTitle(drawerTitle);
                }
                getDelegate().invalidateOptionsMenu(); // creates call to onPrepareOptionsMenu()
            }
        };
        drawerLayout.addDrawerListener(drawerToggle);

        if (savedInstanceState == null) {
            if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                String[] permissions = new String[]{Manifest.permission.ACCESS_FINE_LOCATION};
                if (Build.VERSION.SDK_INT == Build.VERSION_CODES.Q) {
                    permissions = new String[]{Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_BACKGROUND_LOCATION};
                } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                    permissions = new String[]{Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.BLUETOOTH_SCAN};
                }
                ActivityCompat.requestPermissions(this, permissions, 1);
            } else {
                selectItem(0);
            }
        }
        // preload the Google map to avoid a 3 second delay when switch to the demo
        if (useGoogleMaps()) {
            this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    com.google.android.gms.maps.MapView mapView = new com.google.android.gms.maps.MapView(MainActivity.this);
                    mapView.onCreate(null);
                }
            });
        }

        // check if we can load the deeplink and open associated placemark
        if (getIntent() != null && getIntent().getData() != null) {
            if (checkUriAndOpenPlacemark(getIntent())) {
                drawerLayout.closeDrawer(drawerList);
                getIntent().setData(null); // reset the intent, so we don't open this every single time
            }
        }
    }

    private boolean checkUriAndOpenPlacemark(Intent intent) {
        if (intent == null || intent.getData() == null || Strings.isNullOrEmpty(intent.getDataString())) {
            return false;
        }

        Uri data = intent.getData();
        String path = data.getPath();

        if (!Strings.isNullOrEmpty(path)) {
            String[] pathPieces = path.split("/");
            for (int i = 0; i < pathPieces.length; i++) {
                // try to open a placemark from deeplink
                if ("placemarks".equals(pathPieces[i]) && i + 1 < pathPieces.length && pathPieces[i + 1] != null) {
                    String placemarkId = pathPieces[i + 1];
                    MapFragment mapFragment = new MapFragment.Builder()
                            .setPlacemarkId(placemarkId)
                            .setMapKey(MAP_KEY)
                            .setAppKey(APP_KEY).build();

                    mapFragment.setMapEventListener(new MapView.MapEventListener() {
                        @Override
                        public void onMapLoadStart() {

                        }

                        @Override
                        public void onMapLoadFinish() {

                        }

                        @Override
                        public void onPlacemarksLoadFinish() {
                            // example: highlight the placemark from deeplink and display callout
                            MapView mapView = mapFragment.getMapView();
                            if (mapView != null && mapView.getAllMarkers() != null) {
                                for (Marker marker : mapView.getAllMarkers()) {
                                    mapView.onMarkerClick(marker);
                                    break;
                                }
                            }
                        }

                        @Override
                        public void onMapRenderFinish() {

                        }

                        @Override
                        public void onMapLoadFail(Throwable tr) {

                        }

                        @Override
                        public void onMapTransformChange(Matrix transform) {

                        }

                        @Override
                        public void onLocationUpdated(MeridianLocation location) {

                        }

                        @Override
                        public void onOrientationUpdated(MeridianOrientation orientation) {

                        }
                    });

                    getSupportFragmentManager()
                            .beginTransaction()
                            .replace(R.id.container, mapFragment)
                            .commit();

                    return true;
                }
            }
        }
        // didn't find anything to open
        return false;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main, menu);
        return super.onCreateOptionsMenu(menu);
    }

    /* Called whenever we call invalidateOptionsMenu() */
    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        // If the nav drawer is open, hide action items related to the content view
        boolean drawerOpen = drawerLayout.isDrawerOpen(drawerList);
        menu.findItem(R.id.action_settings).setVisible(!drawerOpen);
        return super.onPrepareOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {

        // The action bar home/up action should open or close the drawer.
        // ActionBarDrawerToggle will take care of this.
        if (drawerToggle.onOptionsItemSelected(item)) {
            return true;
        }

        // Handle action buttons
        if (item.getItemId() == R.id.action_settings) {
            getSupportFragmentManager()
                    .beginTransaction()
                    .replace(R.id.container, new SettingsFragment())
                    .addToBackStack(null)
                    .commit();
            setTitle(getString(R.string.action_settings));
            return true;
        }
        if (item.getItemId() == R.id.action_manual_report) {
            ACRA.getErrorReporter().handleException(null);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private boolean useGoogleMaps() {
        // verify that the Google maps API key has been set in the manifest
        String apiKey;
        try {
            ApplicationInfo app = this.getPackageManager().getApplicationInfo(this.getPackageName(), PackageManager.GET_META_DATA);
            apiKey = app.metaData.getString("com.google.android.geo.API_KEY");
        } catch (Exception ignore) {
            return false;
        }
        return (!Strings.isNullOrEmpty(apiKey) && !apiKey.equals(getString(R.string.add_google_maps_key)));
    }

    /* The click listener for ListView in the navigation drawer */
    private class DrawerItemClickListener implements ListView.OnItemClickListener {
        @Override
        public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
            selectItem(position);
        }
    }

    private void selectItem(int position) {

        // update the main content by replacing fragments
        final Fragment fragment;

        switch (position) {
            case 0:
                MapFragment.Builder builder = new MapFragment.Builder()
                        .setMapKey(MAP_KEY);
                MapOptions mapOptions = MapOptions.getDefaultOptions();
                // Turn off the overview button (only shown if there is an overview map for the location)
                mapOptions.HIDE_OVERVIEW_BUTTON = true;
                // example: how to set placemark markers text size
                //mapOptions.setTextSize(14);
                builder.setMapOptions(mapOptions);

                // examples: how to set directions options.
                /*DirectionsOptions directionsOptions = DirectionsOptions.getDefaultOptions();
                directionsOptions.DIRECTIONS_CALLOUT_COLOR = ContextCompat.getColor(this, R.color.mr_color_primary);
                directionsOptions.DIRECTIONS_SLIDE_COLOR = ContextCompat.getColor(this, R.color.mr_color_primary);
                directionsOptions.ROUTE_OVERVIEW_BUTTON_COLOR = ContextCompat.getColor(this, R.color.mr_md_gray_600);
                directionsOptions.END_ROUTE_BUTTON_COLOR = ContextCompat.getColor(this, R.color.mr_md_gray_600);
                directionsOptions.ROUTE_OVERVIEW_BUTTON_TEXT_COLOR = (ContextCompat.getColor(this, R.color.mr_white));
                directionsOptions.END_ROUTE_BUTTON_TEXT_COLOR = ContextCompat.getColor(this, R.color.mr_color_primary);
                directionsOptions.REORIENTATION_BANNER_COLOR = ContextCompat.getColor(this, R.color.mr_md_gray_600);
                directionsOptions.DIRECTIONS_INSTRUCTIONS_ON_TOP = true;
                directionsOptions.DIRECTIONS_INSTRUCTIONS_TEXT_COLOR = Color.GRAY;
                builder.setDirectionsOptions(directionsOptions);*/

                final MapFragment mapFragment = builder.build();
                mapFragment.setMapEventListener(new MapView.MapEventListener() {

                    @Override
                    public void onMapLoadFinish() { }

                    @Override
                    public void onMapLoadStart() { }

                    @Override
                    public void onPlacemarksLoadFinish() {
                        // example: how to start directions programmatically
                        /*for (Placemark placemark : mapFragment.getMapView().getPlacemarks()) {
                            if ("APPLE".equals(placemark.getName())) {
                                mapFragment.startDirections(DirectionsDestination.forPlacemarkKey(placemark.getKey()));
                            }
                        }*/
                        // example: highlight the first four placemarks
                        /*
                        ArrayList<Marker> markerList = new ArrayList<>();
                        int index = 0;
                        MapView mapView = mapFragment.getMapView();
                        if (mapView != null && mapView.getAllMarkers() != null) {
                            for (Marker marker : mapView.getAllMarkers()) {
                                markerList.add(marker);
                                index++;
                                if (index >= 4) {
                                    HighlightedMarkers highlightedMarkers = new HighlightedMarkers.Builder(markerList).build();
                                    mapView.commitTransaction(new Transaction.Builder().setAnimationDuration(500).addMarker(highlightedMarkers).build());
                                    break;
                                }
                            }
                        }
                        */
                        // example: change color of first placemark marker
                        /*
                        for (Marker marker : mapFragment.getMapView().getAllMarkers()) {
                            if (marker instanceof FlatPlacemarkMarker) {
                                ((FlatPlacemarkMarker) marker).setTintColor(Color.RED);
                                break;
                            }
                        }
                        */
                    }

                    @Override
                    public void onMapRenderFinish() { }

                    @Override
                    public void onMapLoadFail(Throwable tr) {
                        if (mapFragment.isAdded() && mapFragment.getActivity() != null) {
                            String message = getString(com.arubanetworks.meridian.R.string.mr_error_invalid_map);
                            if (tr != null) {
                                if (tr instanceof VolleyError && ((VolleyError) tr).networkResponse != null && ((VolleyError) tr).networkResponse.statusCode == 401) {
                                    message = "HTTP 401 Error: Please verify the Editor token.";
                                } else if (!Strings.isNullOrEmpty(tr.getLocalizedMessage())) {
                                    message = tr.getLocalizedMessage();
                                }
                            }
                            new AlertDialog.Builder(mapFragment.getActivity())
                                    .setTitle(getString(com.arubanetworks.meridian.R.string.mr_error_title))
                                    .setMessage(message)
                                    .setIcon(android.R.drawable.ic_dialog_alert)
                                    .setPositiveButton(com.arubanetworks.meridian.R.string.mr_ok, null)
                                    .show();
                        }
                    }

                    @Override
                    public void onMapTransformChange(Matrix transform) { }

                    @Override
                    public void onLocationUpdated(MeridianLocation location) { }

                    @Override
                    public void onOrientationUpdated(MeridianOrientation orientation) { }
                });

                mapFragment.setMapClickEventListener(new MapView.MapClickEventListener() {
                    @Override
                    public boolean onLocationButtonClick() {
                        // example of how to override the behavior of the location button
                        // Note that the default behavior (return false) provides animation and error handling
                        final MapView mapView = mapFragment.getMapView();
                        MeridianLocation location = mapView.getUserLocation();
                        if (location != null) {
                            mapView.updateForLocation(location);

                        } else {
                            LocationRequest.requestCurrentLocation(getApplicationContext(), APP_KEY, new LocationRequest.LocationRequestListener() {
                                @Override
                                public void onResult(MeridianLocation location) {
                                    mapView.updateForLocation(location);
                                }

                                @Override
                                public void onError(LocationRequest.ErrorType location) {
                                    // handle the error
                                }
                            });
                        }
                        return true;
                    }
                    @Override
                    public boolean onOverviewButtonClick() { return false; }
                    @Override
                    public boolean onLongPress() { return false; }
                    @Override
                    public boolean onMapClick() { return false; }
                });
                fragment = mapFragment;
                break;
            case 1:
                fragment = NearbyFragment.newInstance(APP_KEY);
                break;
            case 2:
                fragment = SearchFragment.newInstance(APP_KEY);
                break;
            case 3:
                fragment = CustomMarkerFragment.newInstance(MAP_KEY);
                break;
            case 4:
                fragment = TriggersFragment.newInstance(APP_KEY);
                break;
            case 5:
                fragment = ScrollingFragment.newInstance(MAP_KEY);
                break;
            case 6:
                fragment = LocationFragment.newInstance(APP_KEY);
                break;
            case 7:
                fragment = LocationSharingFragment.newInstance();
                break;
            case 8:
                fragment = SingleMarkerIDFragment.newInstance(APP_KEY, PLACEMARK_UID, null);
                break;
            case 9:
                if (!Strings.isNullOrEmpty(Application.TAG_MAC)) {
                    fragment = TagsFragment.newInstance(MAP_KEY, Application.TAG_MAC);
                } else {
                    fragment = TagsFragment.newInstance(MAP_KEY);
                }
                break;
            case 10:
                fragment = LocationSharingMapFragment.newInstance(MAP_KEY);
                break;
            case 11:
                fragment = OverlayMarkerFragment.newInstance(MAP_KEY);
                break;
            case 12:
                fragment = new MapViewFragment();
                break;
            case 13:
                fragment = new GoogleMapsFragment();
                // verify that the google maps API key has been set in the manifest
                if (!useGoogleMaps()) {
                    new AlertDialog.Builder(this)
                            .setMessage("You need to provide a valid Google Maps API key in the manifest.")
                            .setPositiveButton("OK", null)
                            .show();
                    return;
                }
                break;
            case 14:
                // placemark deeplink intent sample
                String urlScheme = getString(R.string.url_scheme);

                PackageManager packageManager = getApplicationContext().getPackageManager();
                Intent intent = packageManager.getLaunchIntentForPackage(getApplicationContext().getPackageName());
                //placemarks deeplink uri format: com.arubanetworks.meridian-samples://12345/placemarks/34567_67890
                String uri = urlScheme + "://" + APP_KEY.getId() + "/placemarks/" + MAP_KEY.getId() + "_" + PLACEMARK_UID;

                if (intent != null) {
                    intent.setAction(Intent.ACTION_VIEW);
                    intent.setData(Uri.parse(uri));
                    getApplicationContext().startActivity(intent);
                }
                return;
            default:
                return;
        }

        getSupportFragmentManager()
                .beginTransaction()
                .replace(R.id.container, fragment)
                .commit();

        setTitle(getResources().getStringArray(R.array.section_titles)[position]);
        drawerLayout.closeDrawer(drawerList);
    }

    @Override
    public void setTitle(CharSequence title) {
        this.title = title;
        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.setTitle(title);
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            // now request background permissions, should use a fancy UI for this
            if (permissions[0].equals(Manifest.permission.ACCESS_FINE_LOCATION) && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                String[] permissions2 = new String[]{Manifest.permission.ACCESS_BACKGROUND_LOCATION};
                ActivityCompat.requestPermissions(this, permissions2, 1);
                return;
            }
        }
        selectItem(0);
    }

    /**
     * When using the ActionBarDrawerToggle, you must call it during
     * onPostCreate() and onConfigurationChanged()...
     */
    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);
        // Sync the toggle state after onRestoreInstanceState has occurred.
        drawerToggle.syncState();
    }

    @Override
    public void onConfigurationChanged(@NonNull Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        // Pass any configuration changes to the drawer toggles
        drawerToggle.onConfigurationChanged(newConfig);
    }

}
